<?php
 /* credits.php -- provides an on-page credits block
  * Copyright 2004, Evan Prodromou <evan@pigdog.org>.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

if (!defined('PmWiki')) exit();
define(CREDITS_VERSION, '0.3');
$CreditBlockVersion = 'deprecated, use "CREDITS_VERSION".'; # deprecated

SDV($CreditLimit, 2);
SDV($CreditListToText, 'CreditListToText');
SDV($CreditAuthorLinks, 1);
SDV($CreditLanguage, "EN"); # ISO 639-1 Language Codes.
SDV($CreditLastFmt,         "Last edited by \$last.<br />");
SDV($CreditFirstFmt,        "Originally by \$first.<br />");
SDV($CreditOthersFmt,       "Based on work by \$others.<br />");
SDV($CreditAnonymous,       "an anonymous user of $WikiTitle");
SDV($CreditAnonymousPlural, "anonymous users of $WikiTitle");
SDV($CreditAnd, 'and ');
SDV($CreditComma, ', ');

function CreditGetPageContributers($pagename='') {
    $authors    = array();
    $all_others = array();
    $page       = RetrieveAuthPage($pagename,'read');
    if (!$page) return '';
    Lock(0);
    foreach ($page as $k => $v) {
		if (preg_match('/^author\:\d+/', $k)) {
            $authors[$k] = $v;
        }
    }
    $first  = array_shift($authors); # Original Author
    $latest = array_pop($authors);   # Last Contributer
    foreach ($authors as $k => $v) { # All Other Contributers
        $all_others[CreditAnonymousAuthor($v)]++; 
    }
    if ($all_others[$CreditAnonymous] > 1) {
        $all_others[$CreditAnonymousPlural] = 1;
        unset($all_others[$CreditAnonymous]);
    }
    return array(
        'author' => CreditAnonymousAuthor($first),
        'latest' => CreditAnonymousAuthor($latest), 
        'others' => array_keys($all_others)
    );
}
function CreditAnonymousAuthor($author) {
    Global $CreditAnonymous, $CreditAuthorLinks, $AuthorGroup, $ScriptUrl;
    Global $SpaceWikiWords, $AsSpacedFunction;
    $authorname = ($SpaceWikiWords) 
        ? $AsSpacedFunction($author) : $author;
    return (strlen($author) == 0)
        ? $CreditAnonymous 
        : ($CreditAuthorLinks) 
            ? "<a href=\"$ScriptUrl/$AuthorGroup/$author\">$authorname</a>" 
            : $author;
}
function CreditMakeCreditBlock($pagename) {
    global $ScriptDir, $DefaultGroup, $DefaultName;
    require_once('cookbook/credits/language.php');
    global $CreditFirstFmt, $CreditLastFmt, $CreditOthersFmt;
    global $CreditLimit, $CreditListToText;
    $block = '';
    if ($CreditLimit != 0) {
        if ($pagename == '') $pagename = "$DefaultGroup/$DefaultName";
        $authors = CreditGetPageContributers($pagename);
        if ($authors == '')  return '';
	    $first   = str_replace('$first', $authors['author'], $CreditFirstFmt);
	    $last    = str_replace('$last',  $authors['latest'], $CreditLastFmt);
        $contrib = str_replace(
            '$others', $CreditListToText($authors['others']), $CreditOthersFmt
        );
        $block = ($CreditLimit == 1 || count($authors['others']) == 0) 
            ? ($authors['last'] == $authors['first'])
                ? $first : join(' ', array($last, $first))
            : (count($authors['others']) == 1 
                && $authors['others']['0'] == $authors['author'])
                ? join(' ', array($last, $first))
                : join(' ', array($last, $contrib, $first));
    }
    return $block;
}
    $CreditBlock = CreditMakeCreditBlock($pagename);
?>
